program matmultimes
    implicit none
    integer :: starttime
    real :: looptime, matmultime, sgemmtime
    real, dimension(:,:), allocatable :: a, b, c
    character(len=30) :: arg
    integer :: n
    integer :: i, j, k

    if (command_argument_count() < 1) then
        n = 1000
    else
        call get_command_argument(1,arg)
        read (arg,*), n
        if (n < 5 .or. n > 100000) n = 1000
    endif
    
    print *, 'Experiment with matrix size ', n
    print *, 'Times in seconds.'

    allocate(a(n,n))
    allocate(b(n,n))
    allocate(c(n,n))
    call random_number(a)
    call random_number(b)

    call tick(starttime)
    do j=1,n
        do i=1,n
            c(i,j) = 0.
            do k=1,n
                c(i,j) = c(i,j)+a(i,k)*b(k,j)
            enddo
        enddo
    enddo
    looptime = tock(starttime)

    call tick(starttime)
    c = matmul(a,b)
    matmultime = tock(starttime)

    call tick(starttime)
    call sgemm('N','N',n,n,n,1.,a,n,b,n,0.,c,n)
    sgemmtime = tock(starttime)


    print *, 'Triple-loop time:      ', looptime
    print *, 'matmul intrinsic time: ', matmultime
    print *, 'SGEMM lapack call time:', sgemmtime
    
    deallocate(a,b,c)

contains

subroutine tick(t)
    integer, intent(OUT) :: t

    call system_clock(t)
end subroutine tick

! returns time in seconds from now to time described by t
real function tock(t)
    integer, intent(in) :: t
    integer :: now, clock_rate

    call system_clock(now,clock_rate)

    tock = real(now - t)/real(clock_rate)
end function tock

end program matmultimes
