       program diffuse
       implicit none
!
! simulation parameters
!
       integer, parameter :: totpoints=1000
       integer :: locpoints, start
       real, parameter    :: xleft=-12., xright=+12.
       real, parameter    :: kappa=1.
       integer, parameter :: nsteps=100000
!
! the calculated temperature, and the known correct
! solution from theory
!
       real, allocatable :: x(:)
       real, allocatable :: temperature(:,:)[:]
       real, allocatable :: theory(:)

       integer :: old=1, new=2
       integer :: step
       integer :: i
       integer :: unitno
       real :: time
       real :: dt, dx
       real :: error

       character(len=3) :: imgstr
!
!  parameters of the original temperature distribution
!
       real, parameter :: ao=1., sigmao = 1.
       real :: a, sigma
       integer :: leftneigh, rightneigh

!
! set parameters
!
       dx = (xright-xleft)/(totpoints-1)
       dt = dx**2 * kappa/10.

!
! find local number of points and where we start in the
! global domain
!  
       locpoints = (totpoints)/num_images()
       start = locpoints*(this_image()-1)
       leftneigh = this_image()-1
       rightneigh= this_image()+1
!
! prefix for our files
!
      write(imgstr,'(I03)') this_image()

! 
! allocate data, including ghost cells: old and new timestep
! theory doesn't need ghost cells, but we include it for simplicity
!
       allocate(temperature(locpoints+2,2)[*])
       allocate(theory(locpoints+2))
       allocate(x(locpoints+2))
!
!  setup initial conditions
!
       time = 0.
       forall (i=1:locpoints+2)
           x(i)     = xleft + (start+i-1)*dx
           temperature(i,old)   = ao*exp(-(x(i))**2 / (2.*sigmao**2))
           theory(i)= ao*exp(-(x(i))**2 / (2.*sigmao**2))
       end forall

       open(newunit=unitno,file=imgstr//'-ics.txt')
       do i=2,locpoints+1
          write(unitno,'(3(F8.3,3X))'),x(i),temperature(i,old), theory(i)
       enddo
       close(unitno)

!
!  evolve
!
       do step=1, nsteps
!
! boundary conditions: keep endpoint temperatures fixed.
!
           temperature(1,new) = temperature(1,old)
           temperature(locpoints+2,new) = temperature(locpoints+2,old)

! exchange boundary information
           sync all
           if (this_image() /= 1) then
              temperature(1,old) = &
                 temperature(locpoints+1,old)[leftneigh]
           endif
           if (this_image() /= num_images()) then
              temperature(locpoints+2,old) = &
                  temperature(2,old)[rightneigh]
           endif
           sync all
!
! update solution
!
           forall (i=2:locpoints+1)
               temperature(i,new) = temperature(i,old) + &
                     dt*kappa/(dx**2) * (                &
                          temperature(i+1,old) -         &
                        2*temperature(i,  old) +         & 
                          temperature(i-1,old)           &
                     )
           end forall
           time = time + dt

! 
! update correct solution
!
           sigma = sqrt(2.*kappa*time + sigmao**2)
           a = ao*sigmao/sigma
           do i=1, locpoints+2
                theory(i) = a*exp(-(x(i))**2 / (2.*sigma**2))
           enddo
           error = sqrt(sum((theory(2:locpoints+1) - temperature(2:locpoints+1,new))**2))
           if (this_image() == 1) then
               print *, 'Step = ', step, 'Time = ', time, ' Err = ', error
           endif

           old = new
           new = new + 1
           if (new == 3) new = 1
       enddo

       open(newunit=unitno,file=imgstr//'-output.txt')
       do i=2,locpoints+1
          write(unitno,'(3(F8.3,3X))'),x(i),temperature(i,new), theory(i)
       enddo
       close(unitno)
 
       deallocate(temperature)
       deallocate(theory)
       deallocate(x)
       end
