       program diffuse
       implicit none
!
! simulation parameters
!
       integer, parameter :: totpoints=1000
       real, parameter    :: xleft=-12., xright=+12.
       real, parameter    :: kappa=1.
       integer, parameter :: nsteps=100000
!
! the calculated temperature, and the known correct
! solution from theory
!
       real, allocatable :: x(:)
       real, allocatable, target :: temperature(:,:)
       real, allocatable :: theory(:)
       real, dimension(:), pointer :: old, new, tmp

       integer :: step
       integer :: i
       integer :: unitno
       real :: time
       real :: dt, dx
       real :: error
!
!  parameters of the original temperature distribution
!
       real, parameter :: ao=1., sigmao = 1.
       real :: a, sigma

!
! set parameters
!
       dx = (xright-xleft)/(totpoints-1)
       dt = dx**2 * kappa/10.

! 
! allocate data, including ghost cells: old and new timestep
! theory doesn't need ghost cells, but we include it for simplicity
!
       allocate(temperature(totpoints+2,2))
       allocate(theory(totpoints+2))
       allocate(x(totpoints+2))
!
!  setup initial conditions
!
       old => temperature(:,1)
       new => temperature(:,2)
       time = 0.
       forall (i=1:totpoints+2)
           x(i)     = xleft + (i-1)*dx
           old(i)   = ao*exp(-(x(i))**2 / (2.*sigmao**2))
           theory(i)= ao*exp(-(x(i))**2 / (2.*sigmao**2))
       end forall

       open(newunit=unitno,file='ics.txt')
       do i=2,totpoints+1
          write(unitno,'(3(F8.3,3X))'),x(i),old(i), theory(i)
       enddo
       close(unitno)

!
!  evolve
!
       do step=1, nsteps
!
! boundary conditions: keep endpoint temperatures fixed.
!
           new(1)           = old(1)
           new(totpoints+2) = old(totpoints+2)
!
! update solution
!
           forall (i=2:totpoints+1)
               new(i) = old(i) + dt*kappa/(dx**2) *  &
                          (old(i+1) - 2*old(i) + old(i-1))
           end forall
           time = time + dt

! 
! update correct solution
!
           sigma = sqrt(2.*kappa*time + sigmao**2)
           a = ao*sigmao/sigma
           do i=1, totpoints+2
                theory(i) = a*exp(-(x(i))**2 / (2.*sigma**2))
           enddo
           error = sqrt(sum((theory(1:totpoints+1) - new(1:totpoints+1))**2))
           print *, 'Step = ', step, 'Time = ', time, ' Err = ', error

           tmp => old
           old => new
           new => tmp
       enddo

       open(newunit=unitno,file='output.txt')
       do i=2,totpoints+1
          write(unitno,'(3(F8.3,3X))'),x(i),new(i), theory(i)
       enddo
       close(unitno)
 
       deallocate(temperature)
       deallocate(theory)
       deallocate(x)
       end
