module integrate

contains

recursive function integratefx(xlo, xhi, f, tol) result(integral)
    ! integrate with trapezoid rule, simpsons rule;
    ! if difference between two is larger than 
    ! relevant tolerance, subdivide region.

    implicit none
    real, intent(in) :: xlo, xhi
    interface
        function f(x)
            implicit none
            real :: f
            real, intent(in) :: x        
        end function f
    end interface
    real, intent(in) :: tol
    real :: integral

    real :: dx, xmid
    real :: simpsons, trapezoid, error

    dx = xhi-xlo
    xmid = (xlo+xhi)/2.
    trapezoid = dx*(f(xlo)+f(xhi))/2.
    simpsons = dx/6.*(f(xlo)+4.*f(xmid)+f(xhi))
    error = abs(trapezoid-simpsons)/(0.5*(trapezoid+simpsons))
    if (error > tol) then
        ! too coarse; subdivide
        integral = integratefx(xlo,xmid,f,tol) + &
                   integratefx(xmid,xhi,f,tol) 
    else
        integral = trapezoid
    endif
end function integratefx

end module integrate
