!
! F77-ized version of a Fortran90 TVD split MHD code
!
!  Original F90 code  copyright (C) 2001,2003, Ue-Li Pen
!  written November 2001 by Ue-Li Pen, pen@cita.utoronto.ca
!
! see http://arxiv.org/abs/astro-ph/0305088
! or http://www.cita.utoronto.ca/~pen/MHD
!
! This code is licencensed under the GPL
!
program hydro
  use pbm, only : outputpbm
  use solver, only : nvars, timestep, idens, initialconditions

  implicit none
  integer :: n,nx,ny
  real  :: t,dt
  integer :: iter
  real, allocatable, dimension(:,:,:) :: u
  character(len=30) :: arg

  ! get domain size from command line; if nothing
  ! on command line, prompt user

  if (command_argument_count() < 1) then
      print *,'Enter size of domain, in pixels '
      do 
          read *, n
          if (n < 2 .or. n>500) then
            print *, 'invalid n = ', n, ' try again.'
          else
            exit
      endif
      enddo 
  else
      call get_command_argument(1, arg)
      read( arg,'(I30)'), n
      if (n < 2 .or. n > 500) then
         print *,'invalid n = ', trim(arg), ' using 100'
         n = 100
      endif
  endif
  nx = n
  ny = n
  allocate(u(nvars,nx,ny))

  call initialconditions(u)
  call outputpbm(u,'ics.pbm',idens)
  t=0
  timesteps: do iter=1,nx*6
      call timestep(u,dt)
      t = t + 2*dt
      if (mod(iter,10).eq.1) print *, iter, 'dt = ', dt, ' t = ', t
  end do timesteps
  call outputpbm(u,'dens.pbm',idens)

  deallocate(u)
end
