C
C F77-ized version of a Fortran90 TVD split MHD code
C
C  Original F90 code  copyright (C) 2001,2003, Ue-Li Pen
C  written November 2001 by Ue-Li Pen, pen@cita.utoronto.ca
C
C see http://arxiv.org/abs/astro-ph/0305088
C or http://www.cita.utoronto.ca/~pen/MHD
C
C This code is licencensed under the GPL
C
      program hydro
      integer n,nx,ny
      parameter (n=100,nx=n,ny=n)
      real t,dt
      integer iter
      external cfl
      real cfl
      integer idens, iener, imomx, imomy, nvars
      data idens/1/, iener/4/, imomx/2/, imomy/3/, nvars/4/
      real gamma
      data gamma/1.666666667/
      common /consts/ idens, iener, imomx, imomy, gamma, nvars
      real u(4,nx,ny) 
      real ut(4,nx,ny) 

      call initialconditions(u,nx,ny)
      call outputpbm('ics.pbm',u,idens,nvars,nx,ny)
      t=0
      do 10 iter=1,nx*6
         dt=0.5*cfl(u,nx,ny)

C the x sweep
         call xsweep(u,nx,ny,dt)

C the y sweeps
         call xytranspose(ut,u,nx,ny)
         call xsweep(ut,ny,nx,dt)
         t = t + dt
         call xsweep(ut,ny,nx,dt)
C 2nd x sweep
         call xytranspose(u,ut,ny,nx)
         call xsweep(u,nx,ny,dt)
         t = t + dt
         if (mod(iter,10).eq.1) print *, iter, 'dt = ', dt, ' t = ', t
 10   continue
      call outputpbm('dens.pbm',u,idens,nvars,nx,ny)
      end

      subroutine color(val, lo, hi, rgb)
        real val, lo, hi
        integer rgb(3)

        real scaleval 

        scaleval = (val-lo)/(hi-lo) * 255.
        rgb(1) = scaleval*3
        if (rgb(1) .gt. 255) rgb(1) = 255
        rgb(2) = scaleval
        rgb(3) = 0
      end

      subroutine outputpbm(filename,var,varnum,nvars,nx,ny)
        character(*) filename
        integer varnum
        integer nvars
        real var(nvars,nx,ny)
        integer rgb(3,nx,ny)
   
        real lo, hi
        integer i,j,c
        lo = 1.e+9
        hi = -1.e+9

        do 400 i=1,nx
          do 410 j=1,ny
             lo = min(lo,var(varnum,i,j)) 
             hi = max(hi,var(varnum,i,j)) 
 410      continue
 400    continue

        do 401 i=1,nx
          do 411 j=1,ny
             call color(var(varnum,i,j), lo, hi, rgb(1,i,j))
 411      continue
 401    continue

        open(44,file=filename)
        write(44,'(A)') 'P3'
        write(44,'(2(1x,i4))') nx, ny
        write(44,'(i3)'), 255
        do 430 j=ny,1,-1
           write(44,'(12(i4,1x))') ((rgb(c,i,j),c=1,3), i=1,nx)
 430    continue
        close(44)
      end

       real function cfl(u,nx,ny)
         integer idens, iener, imomx, imomy, nvars
         real gamma
         common /consts/ idens, iener, imomx, imomy, gamma, nvars
         real u(nvars,nx,ny)
         real v,p,c
         integer i,j
         
         c = 0.
         do 440 j=1,ny
            do 440 i=1,nx
               v=max(abs(u(imomx,i,j)),abs(u(imomy,i,j)))/u(idens,i,j)
               p=(u(iener,i,j)-(u(imomx,i,j)**2+u(imomy,i,j)**2)/
     &                          u(idens,i,j)/2)*(gamma-1)
               c=max(c,v+sqrt(abs((gamma*p)/u(idens,i,j))))
 440     continue
         cfl=1./c
       end 

      subroutine initialconditions(u,nx,ny)
        integer idens, iener, imomx, imomy, nvars
        real gamma
        common /consts/ idens, iener, imomx, imomy, gamma, nvars
        integer nx,ny
        real u(nvars,nx,ny)
        integer i,j
        real backgrounddens, projdens, projvel
        parameter (backgrounddens=1.,projdens=50.,projvel=4.)

        do 200 i=1,nx
          do 210 j=1,ny
            u(imomy,i,j)=0
            r = sqrt((j*1.0-ny/2.)**2 + (i*1.0-nx/2.)**2)
            if (r .lt. 0.1*sqrt(nx*nx*1.+ny*ny*1.)) then
                u(idens,i,j) = projdens
                u(imomx,i,j) = projdens*projvel
                u(iener,i,j) =  0.5*(projdens*projvel*projvel) +
     &                          (1./((gamma-1.)*projdens))
            else
                u(idens,i,j) = backgrounddens
                u(imomx,i,j) = backgrounddens*0.
                u(iener,i,j) =  0.5*(backgrounddens*0.*0.) +
     &                          (1./((gamma-1.)*backgrounddens))
            endif
 210      continue
 200    continue
        return
      end 

      subroutine xsweep(u,nx,ny,dt)
        integer idens, iener, imomx, imomy, nvars
        real gamma
        common /consts/ idens, iener, imomx, imomy, gamma, nvars
        integer nx,ny
        real u(nvars,nx,ny),dt
        integer j

        do 50, j=1,ny
           call tvd1d(u(1,1,j),nx,dt)
 50     continue
      end 

      subroutine tvd1d(u,n,dt)
!       Kurganov scheme with 2nd order Runge-Kutta
        integer idens, iener, imomx, imomy, nvars
        real gamma
        common /consts/ idens, iener, imomx, imomy, gamma, nvars
        integer n
        real dt
        real u(nvars,n)

        real v(nvars,n), u1(nvars,n), wr(nvars,n), wl(nvars,n)
        real dfrp, dfrm, dflm, dflp
        real dfl(nvars,n), dfr(nvars,n)
        real flux(nvars,n)
        real c
        integer i, var
      
        call hydroflux(v,c,u,n)
        do 60 i=1,n
            do 62 var=1,nvars
                wr(var,i)=u(var,i)+v(var,i)
                wl(var,i)=u(var,i)-v(var,i)
 62         continue
 60     continue
        do 64 i=1,n
            irshift=mod(n+(i-1)+1,n)+1
            do 66 var=1,nvars
                flux(var,i)=c*(wr(var,i)-wl(var,irshift))/2.
 66         continue
 64     continue
        do 68 i=1,n
            ilshift=mod(n+(i-1)-1,n)+1
            do 70 var=1,nvars
                u1(var,i)=u(var,i)-(flux(var,i)-flux(var,ilshift))*dt/2.
 70         continue
 68     continue

        call hydroflux(v,c,u1,n)
        do 72 i=1,n
            do 74 var=1,nvars
                wr(var,i)=u1(var,i)+v(var,i)
                wl(var,i)=u1(var,i)-v(var,i)
 74         continue
 72     continue
        do 76 i=1,n
            irshift=mod(n+(i-1)+1,n)+1
            ilshift=mod(n+(i-1)-1,n)+1
            do 78 var=1,nvars
                dfrp = c*(wr(var,irshift)-wr(var,i))/2.
                dfrm = c*(wr(var,i)-wr(var,ilshift))/2.
                dfr(var,i) =0.
                if (dfrp*dfrm .gt. 0) then
                   dfr(var,i)=2*dfrp*dfrm/(dfrp+dfrm)
                endif
 78         continue
 76     continue
        do 80 i=1,n
            irshift=mod(n+(i-1)+1,n)+1
            irrshift=mod(n+(i-1)+2,n)+1
            do 82 var=1,nvars
                dflp = c*(wl(var,irshift)-wl(var,irrshift))/2.
                dflm = c*(wl(var,i)-wl(var,irshift))/2.
                dfl(var,i) =0.
                if (dflp*dflm .gt. 0) then
                   dfl(var,i)=2*dflp*dflm/(dflp+dflm)
                endif
 82         continue
 80     continue
        do 90 i=1,n
           irshift=mod(n+(i-1)+1,n)+1
           do 90 var=1,nvars
               flux(var,i)=(c*wr(var,i)-c*wl(var,irshift)+
     &                      (dfr(var,i)-dfl(var,i)))/2.
 90     continue
        do 92 i=1,n
           ilshift=mod(n+(i-1)-1,n)+1
           do 92 var=1,nvars
              u(var,i)=u(var,i)-(flux(var,i)-flux(var,ilshift))*dt
 92     continue
        return
      end 
      
      subroutine hydroflux(v,c,u,n)
        integer n
        real c
        ! locals
        real vx(n), p(n)
        integer idens, iener, imomx, imomy, nvars
        real gamma
        common /consts/ idens, iener, imomx, imomy, gamma, nvars
        integer i
        real v(nvars,n),u(nvars,n)
      
        c = 0.
        do 300 i=1,n
          vx(i)=u(imomx,i)/u(idens,i)
          p(i)=u(iener,i)-(u(imomx,i)**2+u(imomy,i)**2)/u(idens,i)/2
          p(i)=p(i)*(gamma-1)
          v(idens,i)=u(imomx,i)
          v(imomx,i)=u(imomx,i)*vx(i)+p(i)
          v(imomy,i)=u(imomy,i)*vx(i)
          v(iener,i)=(u(iener,i)+p(i))*vx(i)
          c=max(c, abs(vx(i))+sqrt(abs(gamma*p(i)/u(idens,i))))
 300    continue
        if (c .gt. 0) then
           do 310 i=1,n
              v(idens,i) = v(idens,i)/c
              v(imomx,i) = v(imomx,i)/c
              v(imomy,i) = v(imomy,i)/c
              v(iener,i) = v(iener,i)/c
 310       continue
        endif
      end 
      
      subroutine xytranspose(ut,u,nx,ny)
        integer idens, iener, imomx, imomy, nvars
        real gamma
        common /consts/ idens, iener, imomx, imomy, gamma, nvars
        integer nx,ny
        real u(nvars,nx,ny)
        real ut(nvars,ny,nx)
        integer i,j
      
        do 100 j=1,ny
           do 110 i=1,nx
              ut(idens,j,i)=u(idens,i,j)
              ut(imomx,j,i)=u(imomy,i,j)
              ut(imomy,j,i)=u(imomx,i,j)
              ut(iener,j,i)=u(iener,i,j)
 110       continue 
 100    continue
      end 
